import { select, json, tree, event, hierarchy, zoom, linkHorizontal } from 'd3';

const svg = select('svg');

const width = document.body.clientWidth;
const height = document.body.clientHeight;

const margin = {top:0, right:100, bottom:0, left:50};

const innerWidth = 1500;
const innerHeight = 1200;

const treeLayout = tree().size([innerHeight, innerWidth]);


const zoomG = svg
    .attr('width', width)
    .attr('height', height)
	.append('g');

const g = zoomG.append('g')
		.attr('transform', `translate(${margin.left}, ${margin.top})`);


svg.call(zoom().on('zoom', () => {
	g.attr('transform', event.transform)
}));

json('protocol.json')
	.then(data => {
  	const boxHeight = 18;
    const boxPadding = 8;
    const fontSize = parseFloat(window.getComputedStyle(document.body).fontSize);
  	const root = hierarchy(data);
  	const links = treeLayout(root).links();
  	
  	// If the text has only 2 chars, return 3 so the box has a better size
  	const getTextLength = (d) => d.data.data.id.length === 2 ? 3 : d.data.data.id.length;
    const getBoxWidth = (d) => (getTextLength(d) * 8.5);
    const getBoxX = d => d.children ? d.y - (getBoxWidth(d) / 2) : d.y - 1;
    const getBoxFillColor = (d) =>  d.data.data.protocol ? '#93eabf' : 'lightblue';
    
  	const linkPathGenerator = linkHorizontal()
    		.x(d => d.y)
    		.y(d => d.x);
  	g.selectAll('path').data(links)
  		.enter().append('path')
  		.attr('d', linkPathGenerator);
  
  	g.selectAll('text').data(root.descendants())
  		.enter()
  		.append('rect')
  		.attr('x', d => getBoxX(d))
  		.attr('y', d => d.x - (boxHeight / 2))
  		.attr('stroke', 'black')
  		.attr('stroke-width', .5)
  		.attr('rx', 3)
  	  .attr('fill', getBoxFillColor)
  		.attr('height', boxHeight)
  	  .attr('width', d => getBoxWidth(d));		
  
    g.selectAll('text').data(root.descendants())
  		.enter()
  		.append('text')
  		.attr('x', d => d.y)
  		.attr('y', d => d.x)
  		.attr('dy', '0.35em')
  		.attr('text-anchor', d => d.children ? 'middle' : 'start')
  		.attr('font-size', '.8rem')
      .text(d => d.data.data.id);
})